unit frmPopupList;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, GR32_Image_ml, uMlSkinCtrls, uMlSkinPanel, uMlSkinScrollBar,
  VirtualTrees_ml, uMlSkinVirtualTree, uMlSkinLabel, uMlSkinHtmlLabel,
  uMlSkinEdit, uMlLikeMenuDlg;

type
  PItemDataRec = ^TItemDataRec;

  TItemDataRec = record
    Caption: string;
    Data: Pointer;
  end;

  TPopupListForm = class(TForm)
    pnl1: TMlSkinPanel;
    lbCheckItems: TMlSkinHtmlLabel;
    pnl2: TMlSkinPanel;
    vstList: TMlSkinVirtualStringTree;
    sb1: TMlSkinScrollBar;
    edtFilter: TMlSkinEdit;
    mlkmndlg1: TMlLikeMenuDlg;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure vstListGetNodeDataSize(Sender: TBaseVirtualTree_ml; var
      NodeDataSize: Integer);
    procedure FormShow(Sender: TObject);
    procedure edtFilterChange(Sender: TObject);
    {$IFDEF Unicode}
    procedure vstListGetText(Sender: TBaseVirtualTree_ml; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: string);
    {$ELSE}
    procedure vstListGetText(Sender: TBaseVirtualTree_ml;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    {$ENDIF}
    procedure lbCheckItemsHyperLinkClick(Sender: TObject; LinkName: string);
    procedure vstListChecked(Sender: TBaseVirtualTree_ml; Node: PVirtualNode);
  private
    FItems: TStrings;
    FCheckedItems: TStrings;
    FOnCheckedChanged: TNotifyEvent;
    procedure SetItems(const Value: TStrings);
    procedure SetCheckedItems(const Value: TStrings);
    procedure RefreshList(const AFilter: string);
    procedure CheckedAllItems(const All: Boolean);
    procedure SetOnCheckedChanged(const Value: TNotifyEvent);
    procedure DoChecked;
    { Private declarations }
  public
    { Public declarations }
    property Items: TStrings read FItems write SetItems;
    property CheckedItems: TStrings read FCheckedItems write SetCheckedItems;
    property OnCheckedChanged: TNotifyEvent read FOnCheckedChanged write SetOnCheckedChanged;
  end;

var
  PopupListForm: TPopupListForm;

implementation

{$R *.dfm}

procedure TPopupListForm.CheckedAllItems(const All: Boolean);
var
  Node: PVirtualNode;
begin
  with vstList do
  begin
    try
      BeginUpdate;
      Node := GetFirst;
      while Node <> nil do
      begin
        if All then
          Node.CheckState := csCheckedNormal
        else
        begin
          if Node.CheckState = csCheckedNormal then
            Node.CheckState := csUncheckedNormal
          else
            Node.CheckState := csCheckedNormal
        end;
        Node := GetNext(Node);
      end;
      Invalidate;
    finally
      EndUpdate;
    end;
  end;
end;

procedure TPopupListForm.DoChecked;
var
  Node: PVirtualNode;
  PItem: PItemDataRec;
  k: Integer;
begin
  with vstList do
  begin
    Node := GetFirst;
    if edtFilter.Text = '' then
      FCheckedItems.Clear;
    while Node <> nil do
    begin
      PItem := GetNodeData(Node);
      k := FCheckedItems.IndexOf(PItem^.Caption);
      if Node.CheckState = csCheckedNormal then
      begin
        if k < 0 then
          FCheckedItems.AddObject(PItem^.Caption, PItem^.Data);
      end
      else
      begin
        if (edtFilter.Text <> '') and (k >= 0) then
        begin
          FCheckedItems.Delete(k);
        end;
      end;
      Node := GetNext(Node);
    end;
  end;
  if Assigned(FOnCheckedChanged) then
    FOnCheckedChanged(vstList);
end;

procedure TPopupListForm.edtFilterChange(Sender: TObject);
begin
  RefreshList(edtFilter.Text);
end;

procedure TPopupListForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TPopupListForm.FormCreate(Sender: TObject);
begin
  FItems := TStringList.Create;
  FCheckedItems := TStringList.Create;
end;

procedure TPopupListForm.FormDestroy(Sender: TObject);
begin
  FCheckedItems.Free;
  FItems.Free;
end;

procedure TPopupListForm.FormShow(Sender: TObject);
begin
  RefreshList('');
end;

procedure TPopupListForm.lbCheckItemsHyperLinkClick(Sender: TObject; LinkName: string);
begin
  if LinkName = 'Goback' then
  begin
    mlkmndlg1.Stop;
    Exit;
  end
  else if LinkName = 'Flag1' then
    CheckedAllItems(True)
  else
    CheckedAllItems(False);
  DoChecked;
end;

procedure TPopupListForm.RefreshList(const AFilter: string);
var
  i: Integer;
  Node: PVirtualNode;
  PItem: PItemDataRec;
begin
  with vstList do
  begin
    try
      BeginUpdate;
      Clear;
      for i := 0 to FItems.Count - 1 do
      begin
        if AFilter <> '' then
        begin
          if Pos(UpperCase(AFilter), UpperCase(FItems[i])) = 0 then
            Continue;
        end;
        Node := AddChild(nil);
        PItem := GetNodeData(Node);
        PItem^.Caption := FItems[i];
        PItem^.Data := Pointer(FItems.Objects[i]);
        Node.CheckType := ctCheckBox;
        if FCheckedItems.IndexOf(PItem^.Caption) >= 0 then
          Node.CheckState := csCheckedNormal
        else
          Node.CheckState := csUncheckedNormal;
      end;
    finally
      EndUpdate;
    end;
  end;
end;

procedure TPopupListForm.SetCheckedItems(const Value: TStrings);
begin
  FCheckedItems.Assign(Value);
end;

procedure TPopupListForm.SetItems(const Value: TStrings);
begin
  FItems.Assign(Value);
end;

procedure TPopupListForm.SetOnCheckedChanged(const Value: TNotifyEvent);
begin
  FOnCheckedChanged := Value;
end;

procedure TPopupListForm.vstListChecked(Sender: TBaseVirtualTree_ml;
  Node: PVirtualNode);
begin
  DoChecked;
end;

procedure TPopupListForm.vstListGetNodeDataSize(Sender: TBaseVirtualTree_ml; var
  NodeDataSize: Integer);
begin
  NodeDataSize := SizeOf(TItemDataRec);
end;

{$IFDEF Unicode}
procedure TPopupListForm.vstListGetText(Sender: TBaseVirtualTree_ml; Node:
  PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType; var CellText: string);
{$ELSE}
procedure TPopupListForm.vstListGetText(Sender: TBaseVirtualTree_ml;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
{$ENDIF}
var
  PItem: PItemDataRec;
begin
  PItem := Sender.GetNodeData(Node);
  CellText := PItem^.Caption;
end;

end.

