unit frmSearchList;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, VirtualTrees_ml, GR32_ml, uMlSkinVirtualTree, uMlComm, uMlGR32Lib,
  uGR32_PNG, {$IFDEF UNICODE}pngimage, {$ELSE}pngimage_ml, {$ENDIF} uMlLikeMenuDlg, GR32_Image_ml,
  uMlSkinCtrls, uMlSkinPanel, uMlSkinScrollBar,StdCtrls,
 ExtCtrls, uMlSkinLabel, uMlSkinButton, jpeg;

type
  PTestData = ^TTestDataRec;

  TTestDataRec = record
    Caption: string;
    Count: Integer;
  end;

  TSearchListForm = class(TForm)
    mlkmndlg1: TMlLikeMenuDlg;
    pnl1: TMlSkinPanel;
    pnl2: TMlSkinPanel;
    pnl3: TMlSkinPanel;
    img1: TImage;
    img2: TImage;
    lbl1: TLabel;
    lbl3: TLabel;
    lbl4: TLabel;
    btn1: TMlURLButton;
    pnl4: TMlSkinPanel;
    vdt1: TMlSkinVirtualDrawTree;
    sb1: TMlSkinScrollBar;
    procedure FormCreate(Sender: TObject);
    procedure vdt1DrawNode(Sender: TBaseVirtualTree_ml;
      const PaintInfo: TVTPaintInfo);
    procedure vdt1NodeClick(Sender: TBaseVirtualTree_ml;
      const HitInfo: THitInfo);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure vdt1KeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FHotNodeData: PTestData;
    FKey: string;
    procedure InitTestList;
    function GetHotNodeData: PTestData;
    procedure SetKey(const Value: string);
  public
    { Public declarations }
    property HotNodeData: PTestData read GetHotNodeData;
    property Key: string read FKey write SetKey;

    procedure CreateParams(var Params: TCreateParams) ; override;
  end;

var
  SearchListForm: TSearchListForm;

implementation

const
  CST_NormalNodeHeight = 25;

{$R *.dfm}

procedure TSearchListForm.CreateParams(var Params: TCreateParams);
begin
  inherited;
  Params.ExStyle := Params.ExStyle + WS_EX_NOACTIVATE;
end;

procedure TSearchListForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
  SearchListForm := nil;
end;

procedure TSearchListForm.FormCreate(Sender: TObject);
begin
  FHotNodeData := nil;
  InitTestList;
end;

function TSearchListForm.GetHotNodeData: PTestData;
begin
  Result := FHotNodeData;
end;

procedure TSearchListForm.InitTestList;
const
  ArrTestText: array [0 .. 8] of string = ('iphone6s', 'iphone6', 'iphone6ֻ',
    'iphone6s plus', 'iphone5s', 'iphone6sֻ', 'iphone6ֻĤ', 'iphone6sֻĤ',
    'iphone4');
var
  PIt: PTestData;
  i, n: Integer;
  nd: PVirtualNode;
begin
  Randomize;
  with vdt1 do
  begin
    NodeDataSize := SizeOf(TTestDataRec);
    DefaultNodeHeight := CST_NormalNodeHeight;
    TreeOptions.SelectionOptions := [toFullRowSelect];
    TreeOptions.PaintOptions := [toHideFocusRect, toShowButtons, toShowDropmark,
      toShowRoot, toThemeAware, toUseBlendedImages, toAlwaysHideSelection];
    TreeOptions.MiscOptions := [toAcceptOLEDrop, toFullRepaintOnResize,
      toInitOnSave, toToggleOnDblClick, toWheelPanning, toVariableNodeHeight,
      toEditOnClick];
    // ز
    try
      BeginUpdate;
      Clear;
      n := Length(ArrTestText);
      for i := 0 to n - 1 do
      begin
        nd := AddChild(nil);
        PIt := GetNodeData(nd);
        PIt^.Caption := ArrTestText[i];
        PIt^.Count := Random(2000);
      end;
    finally
      EndUpdate;
    end;
  end;
  if n = 0 then
    Height := 40;
end;

procedure TSearchListForm.SetKey(const Value: string);
begin
  FKey := Value;
  vdt1.Invalidate;
end;

procedure TSearchListForm.vdt1DrawNode(Sender: TBaseVirtualTree_ml;
  const PaintInfo: TVTPaintInfo);
var
  FHotNode: PVirtualNode;
  PIt: PTestData;
  buf: TBitmap32;
  bc: TColor;
  X, Y, tw, th, k: Integer;
  s, t1, t2: string;
begin
  buf := TBitmap32.Create;
  buf.SetSize(GetRectWidth(PaintInfo.CellRect),
    GetRectHeight(PaintInfo.CellRect));
  buf.Canvas.Font.Assign(vdt1.Font);
  FHotNode := vdt1.FocusedNode;
  with vdt1 do
  begin
    PIt := GetNodeData(PaintInfo.Node);
  end;
  with buf.Canvas do
  begin
    Brush.Style := bsClear;
    if FHotNode = PaintInfo.Node then
      bc := $000000B1
    else
      bc := clWhite;
    buf.FillRect(PaintInfo.CellRect.Left, PaintInfo.CellRect.Top,
      PaintInfo.CellRect.Right, PaintInfo.CellRect.Bottom, Color32(bc));
    if FHotNode = PaintInfo.Node then
      Font.Color := clWhite
    else
      Font.Color := clBlack;
    s := PIt^.Caption;
    tw := TextWidth(s);
    th := TextHeight(s);
    k := Pos(UpperCase(FKey), UpperCase(PIt^.Caption));
    Y := PaintInfo.CellRect.Top + (GetRectHeight(PaintInfo.CellRect) -
      th) div 2;
    if k > 0 then
    begin
      t1 := Copy(s, 1, k - 1); // ǰ
      X := PaintInfo.CellRect.Left + 3;
      TextOut(X, Y, t1);
      if FHotNode = PaintInfo.Node then
        Font.Color := $0000FF80
      else
        Font.Color := $000000B1;
      X := X + TextWidth(t1);
      TextOut(X, Y, FKey);
      t2 := Copy(s, k + Length(FKey), Length(s)); // 
      if FHotNode = PaintInfo.Node then
        Font.Color := clWhite
      else
        Font.Color := clBlack;
      X := X + TextWidth(FKey);
      TextOut(X, Y, t2);
    end
    else
    begin
      X := PaintInfo.CellRect.Left + 3;
      TextOut(X, Y, s);
    end;

    if FHotNode = PaintInfo.Node then
      Font.Color := clWhite
    else
      Font.Color := $00999999;
    s := Format('Լ%d', [PIt^.Count]);
    tw := TextWidth(s);
    th := TextHeight(s);
    X := PaintInfo.CellRect.Right - tw - 3;
    Y := PaintInfo.CellRect.Top + (GetRectHeight(PaintInfo.CellRect) -
      th) div 2;
    TextOut(X, Y, s);
  end;
  buf.DrawTo(PaintInfo.Canvas.Handle, 0, 0);
  buf.Free;
end;

procedure TSearchListForm.vdt1KeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
  begin
    with vdt1 do
    begin
      FHotNodeData := GetNodeData(FocusedNode);
      mlkmndlg1.Stop;
    end;
  end;
end;

procedure TSearchListForm.vdt1NodeClick(Sender: TBaseVirtualTree_ml;
  const HitInfo: THitInfo);
begin
  with vdt1 do
  begin
    FHotNodeData := GetNodeData(HitInfo.HitNode);
    mlkmndlg1.Stop;
  end;
end;

end.
