unit uMlLottery;

interface

uses
  Windows, SysUtils, uMlSkinCtrls, Classes, Graphics, GR32_ml, GR32_Layers_ml,
  GR32_Polygons_ml, GR32_VectorUtils_ml, Messages, GR32_Misc_ml, GR32_Text_ml,
  GR32_Transforms_ml, uGR32_PNG, GR32_Image_ml, Controls, Forms,
  GR32_Resamplers_ml, uMlComm, uMlAE, Math, Types;

type
  TTurnDone = procedure(Sender: TObject; const HotIndex: Integer) of object;

  TUpdateLayer = (ulBackground, ulArrow);

  TUpdateLayers = set of TUpdateLayer;

  TMlLottery = class;

  TBaseLayer = class(TCustomLayer)
  private
    FLottery: TMlLottery;
  public
    constructor Create(ALayerCollection: TLayerCollection); override;
    destructor Destroy; override;
  end;

  TStartButtonLayer = class(TBaseLayer)
  private
    FMouseDown: Boolean;
  protected
    function DoHitTest(X, Y: Integer): Boolean; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
      override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Paint(Buffer: TBitmap32); override;
  public
    constructor Create(ALayerCollection: TLayerCollection); override;
  end;

  TArrowLayer = class(TBaseLayer)
  private
    FTransformation: TAffineTransformation;
    FBitmap: TBitmap32;
    FAngle: Single;
    FBitmapCenter: TFloatPoint;
    procedure SetAngle(const Value: Single);
    procedure BitmapChanged(Sender: TObject);
  protected
    procedure AdjustTransformation;
    procedure Paint(Buffer: TBitmap32); override;
  public
    constructor Create(ALayerCollection: TLayerCollection); override;
    destructor Destroy; override;
    property Angle: Single read FAngle write SetAngle;
  end;

  TPrizesDataReader = class(TStringList)
  private
  public
    constructor Create; overload;
    destructor Destroy; override;
    procedure GetText(const Index: Integer; out Caption: string; out X, Y, Angle: Double);
    procedure GetImage(const Index: Integer; out FileName: string; out X, Y,
      Angle: Double);
  end;

  TMlLottery = class(TMlSkinCustomLayerControl)
  private
    FScaleFactor: Double;
    FPrizesData: TPrizesDataReader;
    FDebugMode: Boolean;
    FImagePath: string;
    FOriginPoint: TPoint;
    FStartButton: TStartButtonLayer;
    FArrow: TArrowLayer;
    FHotIndex: Integer;
    FUpdateLayers: TUpdateLayers;
    FPressed: Boolean;
    FOnTurnDone: TTurnDone;
    FDynAngles: TDoubleDynArray;
    procedure WMSize(var Msg: TWMSize); message WM_SIZE;
    procedure WMMouseMove(var Msg: TWMMouseMove); message WM_MOUSEMOVE;
    procedure SetPrizesData(const Value: TPrizesDataReader);
    procedure SetDebugMode(const Value: Boolean);
    procedure SetImagePath(const Value: string);
    procedure SetHotIndex(const Value: Integer);
    procedure SetOnTurnDone(const Value: TTurnDone);
    function GetIndexFromAngle(const Angle: Double): Integer;
    function GetAngleFromIndex(const Index: Integer): Double;
  protected
    procedure DoStartButtonClick(Sender: TObject); virtual;
    procedure DoRotateArrow(const HotIndex: Integer); virtual;
    procedure DoPaintBuffer; override;
  public
    property UpdateLayers: TUpdateLayers read FUpdateLayers write FUpdateLayers;
    property ScaleFactor: Double read FScaleFactor;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property OriginPoint: TPoint read FOriginPoint;
    property ImagePath: string read FImagePath write SetImagePath;
    property PrizesData: TPrizesDataReader read FPrizesData write SetPrizesData;
    property DebugMode: Boolean read FDebugMode write SetDebugMode;
    property HotIndex: Integer read FHotIndex write SetHotIndex;
    property OnTurnDone: TTurnDone read FOnTurnDone write SetOnTurnDone;
  end;

implementation

type
  TATAccess = class(TAffineTransformation);

const
  CST_OriginR = 450;

{ TMlLottery }

procedure RotateBitmap(Bmp: TBitmap32; Degs: Integer; AdjustSize: Boolean;
  BkColor: TColor = clNone; Transparent: Boolean = False); overload;
var
  Tmp: TBitmap32;
  Transformation: TAffineTransformation;
begin
  Tmp := TBitmap32.Create;
  Transformation := TAffineTransformation.Create;
  try
    Transformation.BeginUpdate;
    Transformation.SrcRect := FloatRect(0, 0, Bmp.Width, Bmp.Height);
    Transformation.Translate(-0.5 * Bmp.Width, -0.5 * Bmp.Height);
    Transformation.Rotate(0, 0, -Degs);
    if AdjustSize then
      with Transformation.GetTransformedBounds do
        Tmp.SetSize(Round(Right - Left), Round(Bottom - Top))
    else
      Tmp.SetSize(Bmp.Width, Bmp.Height);
    Transformation.Translate(0.5 * Tmp.Width, 0.5 * Tmp.Height);
    Transformation.EndUpdate;
    Tmp.Clear(Color32(BkColor));
    if not Transparent then
      Bmp.DrawMode := dmTransparent;
    Transform(Tmp, Bmp, Transformation);
    Bmp.Assign(Tmp);
    Bmp.OuterColor := Color32(BkColor);
    if Transparent then
      Bmp.DrawMode := dmTransparent;
  finally
    Transformation.Free;
    Tmp.Free;
  end;
end;

constructor TMlLottery.Create(AOwner: TComponent);
begin
  inherited;
  Width := CST_OriginR;
  Height := CST_OriginR;
  Transparent := True;
  RepaintMode := rmOptimizer;
  FScaleFactor := 1;
  FPrizesData := TPrizesDataReader.Create;
  FArrow := TArrowLayer.Create(Layers);
  FStartButton := TStartButtonLayer.Create(Layers);
  FStartButton.OnClick := DoStartButtonClick;
  FDebugMode := False;
  FPressed := False;
  FHotIndex := 0;
  FUpdateLayers := [ulBackground, ulArrow];
end;

destructor TMlLottery.Destroy;
begin
  SetLength(FDynAngles, 0);
  FArrow.Free;
  FStartButton.Free;
  FPrizesData.Free;
  inherited;
end;

procedure TMlLottery.DoPaintBuffer;
var
  n, k, m: Integer;
  x, y, r, ag, d, tx, ty, tr, ar: Double;
  Points: TArrayOfFloatPoint;
  FixedPts: TArrayOfFixedPoint;
  i: Integer;
  pt: TPoint;
  Bmp32, bmpS: TBitmap32;
  Text32: TText32;
  ttfText: TTrueTypeFont;
  rt: TRect;
  str: string;
begin
  inherited;
  if not (ulBackground in FUpdateLayers) then
  begin
    Exit;
  end;
  if PrizesData.Count = 0 then   //ƷбΪ
    Exit;
  try
    //Բ̵
    bmpS := TBitmap32.Create;
    Bmp32 := Bitmap;
    Bmp32.BeginUpdate;
    r := Width;
    if r > Height then
      r := Height;
    r := r / 2;  //뾶
    x := OriginPoint.x;  //ԭ
    y := OriginPoint.y;
    if FDebugMode then
    begin
      //ڵģʽ£ʾǰԭֵ༭Ʒݡ
      GetCursorPos(pt);
      pt := ScreenToClient(pt);
      pt.x := pt.x - Trunc(x);
      pt.y := pt.y - Trunc(y);
      Bmp32.Textout(0, 0, Format('x=%d,y=%d', [pt.x, pt.y]));
    end;
    //Բ
    Points := Ellipse(x, y, r, r);
    PolygonFS(Bmp32, Points, Color32($00214A9D), pfAlternate, nil);
    r := r - 10 * ScaleFactor;
    Points := Ellipse(x, y, r, r);
    PolygonFS(Bmp32, Points, Color32($00686DFC), pfAlternate, nil);
    r := r - 4 * ScaleFactor;
    Points := Ellipse(x, y, r, r);
    PolygonFS(Bmp32, Points, Color32($00494CF3), pfAlternate, nil);

    //Ĭ13СԲ
    k := 3;
    n := k * PrizesData.Count;
    d := 360 / n;
    ag := 0;
    tr := 7 * ScaleFactor;
    ar := r - tr * 2;
    for i := 0 to n - 1 do
    begin
      FixedPts := GetArcPoints(FloatRect(x - ar, y - ar, x + ar, y + ar), ag, ag + d);
      Points := AFixedToAFloat(FixedPts);
      m := Length(Points) div 2;
      tx := Points[m].x;
      ty := Points[m].y;
      Points := Ellipse(tx, ty, tr, tr);
      if i mod 2 = 0 then
        PolygonFS(Bmp32, Points, Color32($00BBE8FE), pfAlternate, nil)
      else
        PolygonFS(Bmp32, Points, Color32(clWhite), pfAlternate, nil);
      ag := ag + d;
    end;

    //ǻԲ
    r := r - 27 * ScaleFactor;
    Points := Ellipse(x, y, r, r);
    PolygonFS(Bmp32, Points, Color32(clWhite), pfAlternate, nil);

    //Ʒθ
    n := PrizesData.Count;
    d := 360 / n;
    SetLength(FDynAngles, n);
    ag := d + d / 2;  //ʼǶ
    for i := 0 to n - 1 do
    begin
      FixedPts := GetPiePoints(FloatRect(x - r, y - r, x + r, y + r), ag, ag + d);
      if i mod 2 = 0 then
      begin
        PolygonXS(Bmp32, FixedPts, Color32($00DEF1FF), pfAlternate, nil);
      end;
      FDynAngles[i] := ag;
      ag := ag + d;
    end;

    //һ (뻹Żһ£ͨ2TArrayOfFloatPointһ͸ԲҲԴﵽһЧ)
    PolygonFS(Bmp32, Points, Color32(0, 0, 0, 44), pfAlternate, nil);
    //ƷͼƬ
    r := r - 38 * ScaleFactor;
    Points := Ellipse(x, y, r, r);
    PolygonFS(Bmp32, Points, Color32(clWhite), pfAlternate, nil);
    n := PrizesData.Count;
    d := 360 / n;
    ag := d + d / 2;  //ʼǶ
    for i := 0 to n - 1 do
    begin
      FixedPts := GetPiePoints(FloatRect(x - r, y - r, x + r, y + r), ag, ag + d);
      if i mod 2 = 0 then
        PolygonXS(Bmp32, FixedPts, Color32($00DEF1FF), pfAlternate, nil);
      ag := ag + d;
    end;

    Text32 := TText32.Create;
    ttfText := TrueTypeFontClass.Create('΢ź');
    for i := 0 to FPrizesData.Count - 1 do
    begin
      PrizesData.GetImage(i, str, tx, ty, ag);
      str := ImagePath + str;
      if FileExists(str) then
      begin
        LoadPNGintoBitmap32(bmpS, str);
        //ƷͼƬ
        tx := tx * ScaleFactor + x;
        ty := ty * ScaleFactor + y;
        RotateBitmap(bmpS, Trunc(ag), True, clNone, True);
        rt := Rect(Trunc(tx), Trunc(ty), Trunc(tx + 60 * ScaleFactor), Trunc(ty
          + 60 * ScaleFactor));
        bmpS.DrawTo(Bmp32, rt);
      end;
      //жȡƷ֡xꡢyꡢǶֵ磺100Ԫ￨,-48,-156,0
      FPrizesData.GetText(i, str, tx, ty, ag);
      //Ʒ
      tx := tx * ScaleFactor + x;
      ty := ty * ScaleFactor + y;
      ttfText.Size := Trunc(10 * ScaleFactor);   //С
      Text32.Angle := Trunc(ag);
      Text32.Draw(Bmp32, tx, ty, str, ttfText, Color32($00214A9D));
    end;
    Bmp32.EndUpdate;
    bmpS.Free;
    FUpdateLayers := FUpdateLayers - [ulBackground];
  finally
    SetLength(Points, 0);
  end;
end;

procedure TMlLottery.DoRotateArrow(const HotIndex: Integer);
var
  d: Double;
begin
  d := 360 / FPrizesData.Count;
  FArrow.Angle := GetAngleFromIndex(HotIndex) + d * 0.5;
end;

procedure TMlLottery.DoStartButtonClick(Sender: TObject);
var
  i, m, n: Integer;
  DynSpeed: TResultArray;
begin
  if FPressed then
    Exit;
  try
    FPressed := True;
    Randomize;
    n := Trunc(Math.RandomRange(3, 7) * 360 / FScaleFactor); //תȦ
    m := 360 - Random(360);  //Ƕ
    //УС
    DynSpeed := TMlAnimateEasing.GenerateValues(n, n - n div 6, etQuadEaseIn);
    for i := Length(DynSpeed) - 1 downto 0 do
    begin
      FArrow.Angle := DynSpeed[i] + m;
      Application.ProcessMessages;
    end;
    //ݽǶֵн
    FHotIndex := GetIndexFromAngle(m);
    if Assigned(FOnTurnDone) then
      FOnTurnDone(Self, FHotIndex);
  finally
    FPressed := False;
  end;
end;

function TMlLottery.GetAngleFromIndex(const Index: Integer): Double;
var
  d: Double;
begin
  d := 360 / FPrizesData.Count;
  Result := d + d / 2 + d * Index - 90;
end;

function TMlLottery.GetIndexFromAngle(const Angle: Double): Integer;
var
  ag, d: Double;
  i: Integer;
begin
  Result := 0;
  ag := Angle + 90;
  d := 360 / FPrizesData.Count;
  for i := Length(FDynAngles) - 1 downto 0 do
  begin
    if (ag >= FDynAngles[i]) and (ag < FDynAngles[i] + d) then
    begin
      Result := i;
      Break;
    end;
  end;
end;

procedure TMlLottery.SetDebugMode(const Value: Boolean);
begin
  if FDebugMode <> Value then
  begin
    FDebugMode := Value;
    Invalidate;
  end;
end;

procedure TMlLottery.SetHotIndex(const Value: Integer);
begin
  FHotIndex := Value;
  DoRotateArrow(FHotIndex);
end;

procedure TMlLottery.SetImagePath(const Value: string);
begin
  FImagePath := Value;
end;

procedure TMlLottery.SetOnTurnDone(const Value: TTurnDone);
begin
  FOnTurnDone := Value;
end;

procedure TMlLottery.SetPrizesData(const Value: TPrizesDataReader);
begin
  FPrizesData.Assign(Value);
  Invalidate;
end;

procedure TMlLottery.WMMouseMove(var Msg: TWMMouseMove);
begin
  inherited;
  if FDebugMode then
    Invalidate;
end;

procedure TMlLottery.WMSize(var Msg: TWMSize);
var
  r: Double;
begin
  inherited;
  r := Msg.Width;
  if r > Msg.Height then
    r := Msg.Height;
  FScaleFactor := r / CST_OriginR;
  FOriginPoint.X := Width div 2;
  FOriginPoint.Y := Height div 2;
  FUpdateLayers := [ulBackground, ulArrow];
end;

{ TBaseLayer }

constructor TBaseLayer.Create(ALayerCollection: TLayerCollection);
begin
  inherited;
  MouseEvents := False;
  FLottery := TMlLottery(ALayerCollection.Owner);
end;

destructor TBaseLayer.Destroy;
begin

  inherited;
end;

{ TPrizesDataReader }

constructor TPrizesDataReader.Create;
begin
  inherited;

end;

destructor TPrizesDataReader.Destroy;
begin

  inherited;
end;

procedure TPrizesDataReader.GetImage(const Index: Integer; out FileName: string;
  out X, Y, Angle: Double);
var
  str: string;
  k: Integer;
begin
  str := Strings[Index];
  k := Pos('|', str);
  str := Copy(str, k + 1, Length(str));
  with TStringList.Create do
  begin
    CommaText := str;
    FileName := Strings[0];
    X := StrToFloat(Strings[1]);
    Y := StrToFloat(Strings[2]);
    Angle := StrToFloat(Strings[3]);
    Free;
  end;
end;

procedure TPrizesDataReader.GetText(const Index: Integer; out Caption: string;
  out X, Y, Angle: Double);
var
  str: string;
  k: Integer;
begin
  str := Strings[Index];
  k := Pos('|', str);
  str := Copy(str, 1, k - 1);
  with TStringList.Create do
  begin
    CommaText := str;
    Caption := Strings[0];
    X := StrToFloat(Strings[1]);
    Y := StrToFloat(Strings[2]);
    Angle := StrToFloat(Strings[3]);
    Free;
  end;
end;

{ TStartButtonLayer }

constructor TStartButtonLayer.Create(ALayerCollection: TLayerCollection);
begin
  inherited;
  LayerOptions := LOB_VISIBLE or LOB_MOUSE_EVENTS;
  FMouseDown := False;
end;

function TStartButtonLayer.DoHitTest(X, Y: Integer): Boolean;
var
  pt: TPoint;
  rt: TRect;
  r: Integer;
begin
  r := Trunc(40 * FLottery.ScaleFactor);
  pt := Types.Point(X, Y);
  rt := Rect(FLottery.OriginPoint.X - r, FLottery.OriginPoint.Y - r, FLottery.OriginPoint.X
    + r, FLottery.OriginPoint.Y + r);
  Result := Types.PtInRect(rt, pt);
end;

procedure TStartButtonLayer.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;
  FMouseDown := True;
  Changed;
end;

procedure TStartButtonLayer.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited;
  FMouseDown := False;
  Changed;
end;

procedure TStartButtonLayer.Paint(Buffer: TBitmap32);
var
  x, y, r: Double;
  Points: TArrayOfFloatPoint;
  rt: TRect;
  t: Integer;
begin
  try
    Buffer.BeginUpdate;
    x := FLottery.OriginPoint.x;  //ԭ
    y := FLottery.OriginPoint.y;
    //
    if FMouseDown then
      t := 1
    else
      t := 0;
    r := 40 * FLottery.ScaleFactor;
    //Բ
    x := x - 2 + t;
    y := y - 2 + t;
    Points := Ellipse(x, y, r, r);
    PolygonFS(Buffer, Points, Color32($00B9B9F3), pfAlternate, nil);
    x := x + 1 + t;
    y := y + 1 + t;
    Points := Ellipse(x, y, r, r);
    PolygonFS(Buffer, Points, Color32($006064F4), pfAlternate, nil);

    rt := GetBoundsRect(FloatPointToFixedPoint(Points));
    Buffer.Font.Size := Trunc(14 * FLottery.ScaleFactor);
    Buffer.Font.Color := clWhite;
    Buffer.Font.Style := [fsBold];
    Buffer.Font.Name := '΢ź';
    Buffer.Textout(rt, DT_CENTER + DT_VCENTER + DT_SINGLELINE, 'ʼ');
  finally
    SetLength(Points, 0);
    Buffer.EndUpdate;
  end;
end;

{ TArrowLayer }

procedure TArrowLayer.AdjustTransformation;
var
  r, x, y: Double;
begin
  r := 75 * FLottery.ScaleFactor;
  FBitmap.SetSize(Trunc(r * 2), Trunc(r * 2));
  FBitmapCenter := FloatPoint(FBitmap.Width * 0.5, FBitmap.Height * 0.5);
  FTransformation.Clear;
  FTransformation.Translate(-FBitmapCenter.X, -FBitmapCenter.Y);
  FTransformation.Rotate(0, 0, FAngle);
  x := FBitmapCenter.X + (FLottery.Width - FBitmap.Width) / 2;
  y := FBitmapCenter.Y + (FLottery.Height - FBitmap.Height) / 2;
  FTransformation.Translate(x, y);
end;

procedure TArrowLayer.BitmapChanged(Sender: TObject);
begin
  FTransformation.SrcRect := FloatRect(FBitmap.BoundsRect);
  Changed;
end;

constructor TArrowLayer.Create(ALayerCollection: TLayerCollection);
begin
  inherited;
  FBitmap := TBitmap32.Create;
  FBitmap.OnChange := BitmapChanged;
  FTransformation := TAffineTransformation.Create;
  FAngle := 0;
  ForceUpdate := True;
  with FBitmap do
  begin
    BeginUpdate;
    TLinearResampler.Create(FBitmap);
    TCustomResampler(FBitmap.Resampler).PixelAccessMode := pamTransparentEdge;
    DrawMode := dmBlend;
    EndUpdate;
    Changed;
  end;
end;

destructor TArrowLayer.Destroy;
begin
  FTransformation.Free;
  FBitmap.Free;
  inherited;
end;

procedure TArrowLayer.Paint(Buffer: TBitmap32);
var
  Points: TArrayOfFloatPoint;
  x, y, r, w, h: Double;
begin
  try
    AdjustTransformation;
    if ulArrow in FLottery.UpdateLayers then
    begin
      FBitmap.BeginUpdate;
      x := FBitmap.Width / 2;
      y := FBitmap.Height / 2;
      r := 50 * FLottery.ScaleFactor;
    //ͷ
      SetLength(Points, 3);
      w := 30 * FLottery.ScaleFactor;
      h := r + 23 * FLottery.ScaleFactor;

      Points[0] := FloatPoint(x, y);
      Points[1] := FloatPoint(x - w, y);
      Points[2] := FloatPoint(x, y - h);
      PolygonFS(FBitmap, Points, Color32($00686DFC), pfAlternate, nil);

      Points[0] := FloatPoint(x, y);
      Points[1] := FloatPoint(x, y - h);
      Points[2] := FloatPoint(x + w, y);
      PolygonFS(FBitmap, Points, Color32($00494CF3), pfAlternate, nil);

    //ť
      y := FBitmap.Height / 2;
      Points := Ellipse(x, y, r, r);
      PolygonFS(FBitmap, Points, Color32(clWhite), pfAlternate, nil);
      PolylineFS(FBitmap, Points, Color32($00E0E0E5), True);
      r := r - 8 * FLottery.ScaleFactor;
      Points := Ellipse(x, y, r, r);
      PolygonFS(FBitmap, Points, Color32($001C1CD7), pfAlternate, nil);
      FBitmap.EndUpdate;
      FLottery.UpdateLayers := FLottery.UpdateLayers - [ulArrow];
    end;
  finally
    SetLength(Points, 0);
  end;
  Transform(Buffer, FBitmap, FTransformation);
end;

procedure TArrowLayer.SetAngle(const Value: Single);
begin
  if FAngle <> Value then
  begin
    FAngle := Value;
    Changed;
  end;
end;

end.

